from enum import Enum

class GripperType(Enum):
    """
    Description
    -----------

    An enum class that defines the type of gripper being used.
    """
    NoGripper = 0
    Robotiq_2f_85 = 1
